#include <stdint.h>
#include <fstream>
#include <iostream>

#include "imagepcx.h"
#include "color.h"

#pragma pack(push, 1)
struct pcx_header {
	unsigned char Manufacturer;
	unsigned char Version;
	unsigned char Encoding;
	unsigned char BitsPerPixel;
	int16_t Xmin, Ymin;
	int16_t Xmax, Ymax;
	int16_t HDpi;
	int16_t VDpi;
	unsigned char Colormap[48];
	unsigned char Reserved;
	unsigned char NPlanes;
	int16_t BytesPerLine;
	int16_t PaletteInfo;
	int16_t HscreenSize;
	int16_t VscreenSize;
	unsigned char FILLER[54];
};
#pragma pack(pop)

using namespace std;

void loadPCXImage(const char filename[], size_t &nx, size_t &ny, std::vector<unsigned char> &image)
{
	pcx_header header;

	ifstream file(filename, ios::in);
	file.read(reinterpret_cast<char*>(&header), sizeof(pcx_header));

	if (header.Manufacturer != 0x0a) {
		cerr << "No pcx found" << endl;
		return;
	}

	nx = header.Xmax - header.Xmin + 1L;
	ny = header.Ymax - header.Ymin + 1L;

	Color* pic = new Color[nx * ny];

	size_t scanlinesize = header.NPlanes * header.BytesPerLine;
	bool usecolormap = false;

	unsigned char *scanline = new unsigned char[scanlinesize];
	switch (header.BitsPerPixel) {
	case 8: {
		unsigned char t, s;
		uint32_t color;
		unsigned char *colormap = new unsigned char[768];
		file.seekg(-769, std::ios_base::end);
		if (12 == file.get())
			usecolormap = true;
		file.read(reinterpret_cast<char*>(colormap), 768);

		file.seekg(128, std::ios_base::beg);

		for (size_t y = 0; y < ny; y++) {
			for (size_t z = 0; z < scanlinesize;) {
				t = file.get();

				if (3 == (t >> 6)) {
					t ^= 0xC0;
					s = file.get();
					for (unsigned int p = 0; p < t; p++) {
						scanline[z] = s;
						z++;
					}
				} else {
					scanline[z] = t;
					z++;
				}
			}
			if (usecolormap != false) {
				for (size_t x = 0; x < nx; x++) {
					color = scanline[x];

					pic[x+y*nx].components.b = colormap[3*color+2];
					pic[x+y*nx].components.g = colormap[3*color+1];
					pic[x+y*nx].components.r = colormap[3*color];
					pic[x+y*nx].components.a = 0xff;
				}
			} else {
				for (size_t x = 0; x < nx; x++) {
					pic[x+y*nx].components.b = scanline[x+header.BytesPerLine+header.BytesPerLine];
					pic[x+y*nx].components.g = scanline[x+header.BytesPerLine];
					pic[x+y*nx].components.r = scanline[x];
					pic[x+y*nx].components.a = 0xff;
				}
			}
		}
	}
	break;
	case 1: {
		unsigned char t, s;
		file.seekg(128, std::ios_base::beg);
		unsigned char index = 0, index1b;
		uint32_t colortable[2] = {0x00000000, 0x00ffffff};
		int read = 0;
		unsigned int scanlinepos;


		for (size_t y = 0; y < ny; y++) {
			for (size_t z = 0; z < scanlinesize;) {
				t = file.get();

				if (3 == (t >> 6)) {
					t ^= 0xC0;
					s = file.get();
					for (unsigned p = 0; p < t; p++) {
						scanline[z] = s;
						z++;
					}
				} else {
					scanline[z] = t;
					z++;
				}
			}

			scanlinepos = 0;
			read = 0;
			for (size_t x = 0; x < nx; x++) {
				if (read != 0) {
					index = scanline[scanlinepos];
					scanlinepos++;
					read = 8;
				}
				index1b = 0x01 & (index >> (read - 1));
				pic[x+y*nx].color = colortable[index1b];
				read--;
			}
		}
	}
	break;
	default: {
	}
	break;
	}

	image.assign(reinterpret_cast<char*>(pic), reinterpret_cast<char*>(pic) + nx * ny * 4);
	delete[] pic;
}

