#include "mask.h"
#include <iostream>

using namespace std;

vector<unsigned char> Mask::extract_image(const vector<unsigned char> &image)
{
	vector<unsigned char> newimage(mask_width * mask_height * color_components);
	unsigned int pos = (border_top * image_width + border_left) * color_components;
	unsigned int newpos = 0;
	
	for (unsigned int y = 0; y < mask_height; y++) {
		newimage[newpos + 0] = image[pos + 0];
		newimage[newpos + 1] = image[pos + 1];
		newimage[newpos + 2] = image[pos + 2];
		newimage[newpos + 3] = image[pos + 3];
		newpos += color_components;
		pos += image_width * color_components;
	}

	return newimage;
}

Mask::Mask(const bitset<mask_width * mask_height> &r) :
		mask(r)
{
}

Mask::Mask(vector<unsigned char> image)
{
	vector<unsigned char> newimage = extract_image(image);


	for (unsigned int i = 0; i < mask_width * mask_height; i++) {
		bool isset_color = false;
		for (unsigned j = 2; j < 3; j++) {
			if (newimage[i * color_components + j] != 0) {
				isset_color = true;
			}
		}

		if (isset_color) {
			mask.reset(i);
		} else {
			mask.set(i);
		}

	}
}

void Mask::print() const
{
	bool parity = true;
	for (unsigned int y = 3; y < mask_height; y += 4) {
		if (mask[y] == 0) {
			cout << "0";
		} else {
			parity = !parity;
			cout << "1";
		}
	}
	cout << endl;

	if (parity == false) {
		cerr << "Parity is broken" << endl;
	}
}

bool Mask::operator==(const Mask &x) const
{
	for (unsigned int i = 0; i < mask_width * mask_height; i++) {
		if (mask[i] != x.mask[i]) {
			return false;
		}
	}

	return true;
}

bool Mask::operator<(const Mask &x) const
{
	for (unsigned int i = 0; i < mask_width * mask_height; i++) {
		if (mask[i] < x.mask[i]) {
			return true;
		} else if (mask[i] > x.mask[i]) {
			return false;
		}
	}

	return false;
}

bool Mask::empty() const
{
	if (mask.count() == 0) {
		return true;
	} else {
		return false;
	}
}
